## Steps to Run the Code

1. Create a GCP instance:

    1.1. Select machine type "n1-highmem-8" (8 vCPU, 52 GB RAM).

    1.2. Add NVIDIA Tesla T4 GPU.

    1.3. In the boot disk section, select Ubuntu 18.04 LTS with desired amount of disk (eg 100GB).

2. SSH into the created instance, and install Docker:

        $ sudo apt-get update
        $ sudo apt-get install apt-transport-https ca-certificates curl gnupg lsb-release
        $ curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /usr/share/keyrings/docker-archive-keyring.gpg
        $ echo \
          "deb [arch=amd64 signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/ubuntu \
          $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
        $ sudo apt-get update
        $ sudo apt-get install docker-ce docker-ce-cli containerd.io

3. Verify Docker is installed:

        $ sudo docker run hello-world

4. Add your user to docker group (insert your username where `[user]` is):

        $ sudo usermod -a -G docker [user]

5. Run `grep docker /etc/group` to make sure it was successful. You should see something like `docker:x:999:[user]`.

6. Change user group ID to `docker`:

        $ newgrp docker

7. Create an SSH key using the following:

        $ ssh-keygen -t rsa -b 4096 -C "your_email_address"

  When asked for filename, just hit enter, and then enter a passphrase for your SSH key.

8. Print your public SSH key:

        $ cat .ssh/id_rsa.pub

9. In Github, go to Settings, then "SSH and GPG keys", then "New SSH key" and copy and paste your printed key.

10. Click on Enable SSO and Authorize.

11. On the machine, clone the repository:

        $ git clone git@github.com:verily-src/sensors-ds-python-classifier-2021.git

12. Create a folder `example` in your home directory with several subfolders.

        $ cd ~/
        $ mkdir example
        $ cd example
        $ mkdir training_data test_data model test_outputs

13. Put training and test data *records* directly into *training_data* and *test_data* directories. You can download the training data from the [Challenge website](https://physionetchallenges.org/2021/#data-access).

14. Install CUDA using the instructions [here](https://www.tensorflow.org/install/gpu#install_cuda_with_apt) for Ubuntu 18.04.


15. Install NVIDIA container runtime to make CUDA work with Docker ([source](https://www.celantur.com/blog/run-cuda-in-docker-on-linux/)):

        $ curl -s -L https://nvidia.github.io/nvidia-container-runtime/gpgkey | sudo apt-key add -
        $ distribution=$(. /etc/os-release;echo $ID$VERSION_ID)
        $ curl -s -L https://nvidia.github.io/nvidia-container-runtime/$distribution/nvidia-container-runtime.list |\
            sudo tee /etc/apt/sources.list.d/nvidia-container-runtime.list
        $ sudo apt-get update
        $ sudo apt-get install nvidia-container-runtime

16. Restart Docker:

        $ sudo systemctl stop docker
        $ sudo systemctl start docker

17. Change directory to where the code is, and build a Docker image:

        $ docker build -t image .

  This will take a while.

18. Now run Docker:

        $ docker run -it --gpus all -v ~/example/model:/physionet/model \
            -v ~/example/test_data:/physionet/test_data -v ~/example/test_outputs:/physionet/test_outputs \
            -v ~/example/training_data:/physionet/training_data image bash

19. Inside Docker, run train, test, and evaluation pipelines using the following:

        root@[...]:/physionet# python train_model.py training_data model

        root@[...]:/physionet# python test_model.py model test_data test_outputs

        root@[...]:/physionet# python evaluate_model.py test_data test_outputs

 
## Useful links

* [The PhysioNet/CinC Challenge 2021 webpage](https://physionetchallenges.org/2021/)
* [MATLAB example code for the PhysioNet/CinC Challenge 2021](https://github.com/physionetchallenges/matlab-classifier-2021)
* [Evaluation code for the PhysioNet/CinC Challenge 2021](https://github.com/physionetchallenges/evaluation-2021) 
* [2021 frequently asked questions (FAQ)](https://physionetchallenges.org/2021/faq/) 
* [General frequently asked questions (FAQ)](https://physionetchallenges.org/faq/) 

